clear;close all;clc
set(0,'defaultAxesFontSize',16)
% Parameterek
R  = 1e6;       % Ohm
Cs = 1e-6;      % Farad
U0 = 1;         % Volt
% Allapotvaltozos leiras
A = [-1/R/Cs];
B = [ 1/R/Cs];
C = [-1/R];
D = [1/R];
%%
% Euler sema parametere
hk = 0.1;

% Euler sema kiszamitasa adott idolepessel
% Idointervallum eleje/vege
T_start  = 0;    % sec
T_end  = 5;    % sec
% idovektor letrehozasa
t_vec = T_start:hk:T_end;

%% valtozok inicializalasa
u = ones(1, length(t_vec));
% elorelepo Euler semahoz
x_elore = zeros(length(C), length(t_vec));
y_elore = zeros(1,length(t_vec));
% hatralepo Euler semahoz
x_hatra = zeros(length(C), length(t_vec));
y_hatra = zeros(1,length(t_vec));

% "egysegmatrix"
E = eye(size(A));

%% kezdeti ertekek
x_elore(1,:) = 0;
x_hatra(1,:) = 0;

% Elso lepesben a valasz (a ciklusban az elso lepes ele nem latunk)
y_elore(1) = C*x_elore(:,1) + D*u(1);
y_hatra(1) = C*x_hatra(:,1) + D*u(1);
% ciklus az iterativ szamitashoz
for ii = 2:length(t_vec)
    tk = t_vec(ii);
    x_elore(:,ii) = (E + hk*A)*x_elore(:,ii-1) + hk*B*u(:,ii-1);
    y_elore(ii) = C*x_elore(:,ii) + D*u(ii);
    
    % !!! inv(E-hk*A)* == (E-hk*A)\
    x_hatra(:,ii) = (E-hk*A)\(x_hatra(:,ii-1) + hk*B*u(:,ii));
    y_hatra(ii) = C*x_hatra(:,ii) + D*u(ii);
    
end

%% Analitikus megoldas kiertekelese
dt = 0.001;     % sec
t_vec_a = T_start:dt:T_end;
 
% "analitikus" valasz szamitasa
y_a=U0/R*exp(-t_vec_a./(R*Cs));

y_ha=U0/R*exp(-t_vec./(R*Cs));

e_elore =  abs(y_ha - y_elore).^2 ;
e_hatra =  abs(y_ha - y_hatra).^2;

%% kirajzolas
% osszehasonlitas
figure(1)
plot(t_vec,1e6*y_elore,'k-o','LineWidth',2)
hold on
plot(t_vec,1e6*y_hatra,'b-s','LineWidth',2)
plot(t_vec_a,1e6*y_a,'r--','LineWidth',2)
grid on
xlabel('Ido [s]')
ylabel('i [uA]')
legend('Elorelepo Euler','Hatralepo Euler','analitikus')
title('Elore- es hatralepo Euler sema ')
%
figure(2)
plot(t_vec,e_elore,'k-o','LineWidth',2)
hold on
grid on
plot(t_vec,e_hatra,'b-s','LineWidth',2)
xlabel('Ido [s]')
ylabel('absz. negyzetes hiba')
legend('Elorelepo Euler','Hatralepo Euler')
title('Kozelitesi hiba')